@@ -11,6 +11,7 @@ import pandas as pd
 import json
 import lz4.block
 import struct
+import base64
 
 class HistoricalREST:
     def __init__(self, login_id: str, password: str, url: str, broker_token: str, logger: Logger):  # NO PORT, broker token needed from now on
@@ -76,9 +77,13 @@ class HistoricalREST:
             with self.thread_lock:
                 url = f"{self.url}/{source_bar_type_string}?{unencoded_payload}"
                 response = requests.get(url, headers=headers, params=encoded_payload)
+                if response.status_code == 429:
+                    error = base64.b64decode(response.content).decode('utf-8')
+                    raise TooManyRequestsError(error)
                 hist_data = HistoricalREST.decompress_data(response.content)
         except Exception as e:
             self.logger.error(f"{type(e)} -> {e}")
+            return None
             # json_response = {"success": False, "message": f"({type(e)}) {addn_err_str} -> {e}"}
         hist_data = self.parse_data(hist_data, options)
         return options['processor_to_call'][options['data_type']](hist_data, time_format=options['time_format'])
@@ -112,10 +117,13 @@ class HistoricalREST:
             with self.thread_lock:
                 url = f"{self.url}/{source_bar_type_string}?{unencoded_payload}"
                 response = requests.get(url, headers=headers, params=encoded_payload)
+                if response.status_code == 429:
+                    error = base64.b64decode(response.content).decode('utf-8')
+                    raise TooManyRequestsError(error)
                 hist_data = HistoricalREST.decompress_data(response.content)
-
         except Exception as e:
             self.logger.error(f"{type(e)} -> {e}")
+            return None
             # json_response = {"success": False, "message": f"({type(e)}) {addn_err_str} -> {e}"}
         hist_data = self.parse_data(hist_data, options)
         return options['processor_to_call'][options['data_type']](hist_data, time_format=options['time_format'])
@@ -212,7 +220,7 @@ class HistoricalREST:
                 data_list.append({'time': datetime.strptime(j[0], time_format),
                                   'ltp': float(j[1]),
                                   'volume': int(j[2]),
-                                  'oi': int(j[3]),
+                                  'oi': int(j[3] if len(j[3]) >= 1 else 0),
                                   'bid': float(j[4]),
                                   'bid_qty': int(j[5]),
                                   'ask': float(j[6]),
@@ -221,7 +229,7 @@ class HistoricalREST:
                 data_list.append({'time': datetime.strptime(j[0], time_format),
                                   'ltp': float(j[1]),
                                   'volume': int(j[2]),
-                                  'oi': int(j[3])})
+                                  'oi': int(j[3] if len(j[3]) >= 1 else 0)})
                 continue
         return data_list
 
@@ -251,7 +259,7 @@ class HistoricalREST:
                 data_list.append({'time': datetime.strptime(j[0], time_format),
                                   'ltp': float(j[1]),
                                   'volume': int(j[2]),
-                                  'oi': int(j[3]),
+                                  'oi': int(j[3] if len(j[3]) >= 1 else 0),
                                   'bid': float(j[4]),
                                   'bid_qty': int(j[5]),
                                   'ask': float(j[6]),
@@ -260,7 +268,7 @@ class HistoricalREST:
                 data_list.append({'time': datetime.strptime(j[0], time_format),
                                   'ltp': float(j[1]),
                                   'volume': int(j[2]),
-                                  'oi': int(j[3])})
+                                  'oi': int(j[3] if len(j[3]) >= 1 else 0)})
                 continue
             count = count + 1
         return data_list
@@ -287,8 +295,12 @@ class HistoricalREST:
     def decompress_data(data):
         uncom_length = struct.unpack('<I', data[:4])[0]
         com_length = struct.unpack('<I', data[4:8])[0]
-        dc = lz4.block.decompress( data[8:] , uncom_length ) if com_length != uncom_length else data[8:]
+        dc = lz4.block.decompress( data[8:], uncom_length ) if com_length != uncom_length else data[8:]
         return dc.decode()
 
-
+class TooManyRequestsError(Exception):
+    def __init__(self, msg):
+        self.msg = msg
+    def __str__(self ):
+        return self.msg
 
